#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>

#define NUMBER_OF_THREADS 2
#define N 10

typedef struct 
{
    int id;
    double alpha;
    double *x;
    double *y;
    double *z;
} axpy_t;

void *axpy(void *arg)
{
    axpy_t *mydata = (axpy_t *)arg;
    double *x = mydata->x;
    double *y = mydata->y;
    double *z = mydata->z;
    double alpha = mydata->alpha;
    int myid = mydata->id;

    int i, slice = N/NUMBER_OF_THREADS;
    for ( i = myid*slice; i < (myid+1)*slice; i++ )
        z[i] = alpha * x[i] + y[i];

    pthread_exit(NULL);
}

int main(int argc, char *argv[])
{
    pthread_t threads[NUMBER_OF_THREADS];
    axpy_t data[NUMBER_OF_THREADS];

    // Computes z = alpha * x + y using NUMBER_OF_THREADS threads
    double x[N], y[N], z[N], alpha;
    int status, i;

    alpha = 5.0;
    for( i = 0; i < N; i++ )
    {
        x[i] = i;
        y[i] = 2*i;
    }

    for(i = 0; i < NUMBER_OF_THREADS; i++) {
        // Initialize data structure (pointers to data and thread id)
        data[i].id = i;
        data[i].alpha = alpha;
        data[i].x = x;
        data[i].y = y;
        data[i].z = z;

        // Create and run thread i
        status = pthread_create(&threads[i], NULL, &axpy, (void *)&data[i]);

        // Issue warning if an error occurred
        if (status != 0) {
            printf("Oops. pthread_create returned error code %d\n", status);
            exit(-1);
        }
    }

    // Wait for the threads to complete
    for(i = 0; i < NUMBER_OF_THREADS; i++) {
        pthread_join(threads[i], NULL);
    }

    // Print results. Should output [0, 7, 14, 21, ...]
    printf("[ ");
    for( i = 0; i < N; i++ )
        printf(" %.0f ", z[i] );
    printf("]\n");

    pthread_exit(NULL);
}

