#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>

#define NUMBER_OF_THREADS 2
#define N 100

typedef struct 
{
    int id;
    double *alpha;
    double *x;
    double *y;
    pthread_mutex_t *mutex;
} dot_t;

void *dot(void *arg)
{
    dot_t *mydata = (dot_t *)arg;
    int myid = mydata->id;
    double *alpha = mydata->alpha;
    double *x = mydata->x;
    double *y = mydata->y;
    pthread_mutex_t *mutex = mydata->mutex;

    int i, slice = N/NUMBER_OF_THREADS;
    for ( i = myid*slice; i < (myid+1)*slice; i++ )
    {
        pthread_mutex_lock( mutex );
        *alpha = *alpha + x[i] * y[i];
        pthread_mutex_unlock( mutex );
    }

    pthread_exit(NULL);
}

int main(int argc, char *argv[])
{
    // Computes z = alpha * x + y using NUMBER_OF_THREADS threads
    int status, i;
    double x[N], y[N], alpha;

    pthread_t threads[NUMBER_OF_THREADS];
    dot_t data[NUMBER_OF_THREADS];
    pthread_mutex_t mutex;

    // Initialize data
    for( i = 0; i < N; i++ )
    {
        x[i] = i;
        y[i] = 2*i;
    }
    alpha = 0.0;

    // Initialize mutex
    pthread_mutex_init( &mutex, NULL );

    for(i = 0; i < NUMBER_OF_THREADS; i++) {
        // Initialize data structure (pointers to data and thread id)
        data[i].id = i;
        data[i].alpha = &alpha;
        data[i].x = x;
        data[i].y = y;
        data[i].mutex = &mutex;

        // Create and run thread i
        status = pthread_create(&threads[i], NULL, &dot, (void *)&data[i]);

        // Issue warning if an error occurred
        if (status != 0) {
            printf("Oops. pthread_create returned error code %d\n", status);
            exit(-1);
        }
    }

    // Wait for the threads to complete
    for(i = 0; i < NUMBER_OF_THREADS; i++) {
        pthread_join(threads[i], NULL);
    }

    // Print result
    printf("Dot product: %.2f\n", alpha);

    // Destroy mutex and terminate
    pthread_mutex_destroy( &mutex );
    pthread_exit(NULL);
}

